from dateutil.parser import parse
import sqlite3 as sql
import PySimpleGUI as sg
import random
import datetime


conn = sql.connect("mm.db", check_same_thread=False)

current_user = "Joe"
current_user_id = 1

cursor = conn.cursor()

cursor.execute("SELECT id,name FROM resource")
resources = cursor.fetchall()
id_to_resource = {id: name for id, name in resources}
resource_to_id = {name: id for id, name in resources}

cursor.execute("""
SELECT resource.name,balance FROM bank_account
INNER JOIN resource ON resource.id = bank_account.resource_id
WHERE user_id = ?
""", (current_user_id,))
bank = {name: balance for name,balance in cursor.fetchall()}

cursor.execute("""
SELECT store_item.id,store_item.name,resource.name,price,claim_amount FROM store_item
INNER JOIN resource ON resource.id = currency
""")
store = {item[0]:item[1:] for item in cursor.fetchall()}

cursor.close()

def get_moons():
    staking_sources = {}
    cursor = conn.cursor()

    cursor.execute("SELECT id,created_at FROM staking_source")

    for i,(sid,ts) in enumerate(cursor.fetchall()):
        cursor.execute("""
        SELECT name,init_supply,staking_event.created_at
        FROM resource_well
        INNER JOIN resource ON resource.id = resource_well.resource_id
        LEFT JOIN staking_event ON staking_event.well_id = resource_well.id
        WHERE resource_well.source_id = ?;
        """, (sid,))
        wells = {name: (supply,timestamp) for name,supply,timestamp in cursor.fetchall()}
        staking_sources[sid] = (ts,wells)
    cursor.close()
    return staking_sources

def get_inventory():
    inventory = {}
    cursor = conn.cursor()

    cursor.execute("""
    SELECT inventory_item.id,store_item_id, COUNT(upgrade_event.id) as upgrades
    FROM inventory_item
    LEFT JOIN upgrade_event ON inventory_item.id = upgrade_event.inventory_item_id
    WHERE inventory_item.user_id = ?
    GROUP BY inventory_item.id;
    """, (current_user_id,))

    fall = cursor.fetchall()
    inventory = {item[0]:item[1:] for item in fall}

    cursor.close()
    return inventory

def mint():
    cursor = conn.cursor()
    cursor.execute("SELECT id FROM users WHERE name = ?", (current_user,))
    result = cursor.fetchone()
    if result and result[0]:
        user_id = result[0]
    else:
        raise Exception("User not found: " + user_name)

    rand_hash = "%010x" % random.randrange(16 ** 16)

    cursor.execute('BEGIN')
    try:
        cursor.execute("""
        INSERT INTO staking_source (user_id, address) VALUES (?, ?)
        """, (user_id, f"0x{rand_hash}"))

        source_id = cursor.lastrowid
        for id,_ in resources:
            init_supply = random.randint(50, 200)
            cursor.execute("""
            INSERT INTO resource_well (source_id, resource_id, init_supply) VALUES (?, ?, ?)
            """, (source_id, id, init_supply))

        conn.commit()
    except sqlite3.Error as error:
        print(error)
        conn.rollback()
    finally:
        cursor.close()
        get_moons()

def buy_item(item_id):
    cursor = conn.cursor()

    cursor.execute("""
        INSERT INTO inventory_item (user_id, store_item_id)
        VALUES (?, ?)
    """, (current_user_id, item_id))

    conn.commit()
    cursor.close()

def mine(a,b,user_data):
    source_id,well_name = user_data
    cursor = conn.cursor()

    cursor.execute("""
        INSERT INTO staking_event (source_id, well_id)
        VALUES (?, ?)
    """, (source_id, resource_to_id[well_name]))

    conn.commit()
    get_moons()
    cursor.close()

def upgrade(a,b,user_data):
    ()

def destroy(source_id):
    cursor = conn.cursor()

    cursor.execute("DELETE FROM staking_source WHERE id = ?", (source_id,))

    conn.commit()
    cursor.close()


banks = ""
for name,amount in bank.items():
    banks += f"{name.capitalize()}: {amount} | "

def get_store_ui():
    inventory = get_inventory()
    store_ui = []
    for id,(name,resource,price,claim) in store.items():
        owned = False
        for (store_item_id,_) in inventory.values():
            if id == store_item_id:
                owned = True
        store_ui.append([sg.Text(f"{name}: Mine {claim} {resource.capitalize()}"),
                        sg.Button(button_text=f"Buy {price} {resource[0:3]}",
                                key=f"-BUY-{id}-",
                                disabled=owned)])
    return store_ui

staking_sources = get_moons()
sources_ui = []
for id,source in staking_sources.items():
    wells_ui = []
    for name,(supply,ts) in source[1].items():
        wells_ui.append(sg.Text(name))
    sources_ui.append([sg.Image("moon.png"),
                       sg.Column(layout=[wells_ui]),
                       sg.Button("Upgrade"),
                       sg.Button(f"Destroy-{id}")])

layout = [
    [sg.Text(f"User: {current_user}")],
    [sg.Text(banks, key='-BANKS-')],
    [sg.Button(button_text="Mint Moon", key="-MINT-")],
    [sg.HorizontalSeparator()],
    [sg.Column(get_store_ui(), vertical_alignment='t')],
    [sg.HorizontalSeparator()],
    [sg.Column(layout=sources_ui, size=(1200, 500), scrollable=True, vertical_scroll_only=True)]
]

window = sg.Window("Moon Miner", layout, font='25')

while True:
    event, values = window.read()
    if event == sg.WINDOW_CLOSED:
        break
    elif event.startswith("-BUY-"):
        id = event.split("-")[2]
        buy_item(id)
        window[f"-BUY-{id}-"].update(disabled=True)
    elif event.startswith("Upgrade-"):
        print("Upgrade")
        pass
    elif event.startswith("Destroy"):
        destroy(event.split("-")[1])
    elif event == "-MINT-":
        window['items'].update([[sg.Text("New Text")]])
    elif event.startswith("Mine-"):
        print("Mine")
        pass


window.close()
conn.close()
